#include "windef.h"
#include "define.h"
#include "interfaces.h"
#include "classes.h"
#include <stdexcept>

using namespace std;

D2CBuffer::D2CBuffer() {

	setBufferPointer(NULL);
	setSize(0);

	reset(256);

}

D2CBuffer::D2CBuffer(I32 i32Arg) {

	setBufferPointer(NULL);
	setSize(0);

	if (i32Arg<1)
		return;

	reset(i32Arg);

}

D2CBuffer::D2CBuffer(const BUF bfArg,I32 i32Arg) {

	setBufferPointer(NULL);
	setSize(0);

	if (bfArg==NULL || i32Arg<1)
		return;

	if (!reset(i32Arg))
		return;

	write(0,bfArg,getSize());

}

D2CBuffer::D2CBuffer(const D2IBufferReader *pbrArg) {

	setBufferPointer(NULL);
	setSize(0);

	assign(pbrArg);

}

D2CBuffer::D2CBuffer(const D2CBuffer& bArg) {

	setBufferPointer(NULL);
	setSize(0);

	if (bArg.getSize()<1)
		return;

	if (!reset(bArg.getSize()))
		return;

	write(0,bArg.getBufferPointer(),getSize());

}

D2CBuffer& D2CBuffer::operator=(const D2CBuffer& bArg) {

	if (&bArg==this)
		return *this;

	assign(&bArg);

	return *this;

}


bool D2CBuffer::assign(const D2IBufferReader *pbrArg) {

	if (pbrArg==NULL)
		return false;

	if (!reset(pbrArg->getSize()))
		return false;

	BUF bfTmp=pbrArg->copy();

	write(0,bfTmp,getSize());

	D2Free(bfTmp);

	return true;

}

D2CBuffer::~D2CBuffer() {

	if (getBufferPointer()!=NULL)
		D2Free(getBufferPointer());

}

UI32 D2CBuffer::getClassID() const { return CID_D2CBuffer; }

UI32 D2CBuffer::getObjectType() const {

	I32 i32ID=VER_D2_0 << 24;
	i32ID+=TYPE_BUF << 16;
	i32ID+=ENCODE_NONE;

	return i32ID;

}

UI8 D2CBuffer::getObjectVersion() const { return VER_D2_0; }
UI8 D2CBuffer::getDataType() const { return TYPE_BUF; }
UI16 D2CBuffer::getEncoding() const { return ENCODE_NONE; }

void * D2CBuffer::getInterface(UI32 u32Arg) {

	switch(u32Arg) {

	case IID_D2IObject:
		return (D2IObject *)this;

	case IID_D2IBufferReader:
		return (D2IBufferReader *)this;

	case IID_D2IBufferWriter:
		return (D2IBufferWriter *)this;

	case IID_D2IBufferAdministrator:
		return (D2IBufferAdministrator *)this;

	case IID_D2IFixedBuffer:
		return (D2IFixedBuffer *)this;

	case IID_D2IBuffer:
		return (D2IBuffer *)this;

	}

	return NULL;

}

CSTR D2CBuffer::getCSTR() { return (CSTR)getBufferPointer();}

I32 D2CBuffer::getSize() const { return m_i32BufferSize; }

BUF D2CBuffer::copy() const {

	if (getSize()<1)
		return NULL;

	BUF bfTmp=(BUF)D2Malloc(getSize());

	if (bfTmp==NULL)
		return NULL;

	D2MemCopy(bfTmp,getBufferPointer(),getSize());

	return bfTmp;

}

UI8 D2CBuffer::read(I32 i32Arg) const throw(range_error) {

	if (i32Arg<0 || i32Arg>=getSize())
		throw range_error("ȃCfbNX");

	return getBufferPointer()[i32Arg];

}

I32 D2CBuffer::read(I32 i32Arg1,BUF bfArg,I32 i32Arg2) const {

	if (i32Arg1<0 || i32Arg1>getLastIndex() || bfArg==NULL || i32Arg2<1)
		return 0;

	I32 i32Read=i32Arg2;

	if (i32Arg1+i32Read>getSize())
		i32Read=getSize()-i32Arg1;
	
	if (i32Read<1)
		return 0;

	memcpy(bfArg,getBufferPointer()+i32Arg1,i32Read);

	return i32Read;

}

bool D2CBuffer::equal(const BUF bfArg,I32 i32Arg) const {

	if (i32Arg!=getSize())
		return false;

	for (int i=0;i<=getLastIndex();i++)
		if (bfArg[i]!=read(i))
			return false;

	return true;

}

bool D2CBuffer::equal(const D2IBufferReader *pbrArg) const {

	if (pbrArg==NULL)
		return false;

	if (pbrArg->getSize()!=getSize())
		return false;

	for (int i=0;i<=getLastIndex();i++)
		if (pbrArg->read(i)!=read(i))
			return false;

	return true;

}

bool D2CBuffer::write(I32 i32Arg,UI8 u8Arg) {

	if (i32Arg<0 || i32Arg>getLastIndex())
		return false;

	getBufferPointer()[i32Arg]=u8Arg;

	return true;

}

I32 D2CBuffer::write(I32 i32Arg1,const BUF bfArg,I32 i32Arg2) {

	if (i32Arg1<0 || i32Arg1>getLastIndex() || bfArg==NULL || i32Arg2<1)
		return 0;

	I32 i32Write=i32Arg2;

	if (i32Arg1+i32Write>getSize())
		i32Write=getSize()-i32Arg1;
	
	memcpy(getBufferPointer()+i32Arg1,bfArg,i32Write);

	return i32Write;

}

bool D2CBuffer::fill(I32 i32Arg1,UI8 u8Arg,I32 i32Arg2) {

	if (i32Arg1<0 || i32Arg1>getLastIndex() || i32Arg2<1)
		return false;

	I32 i32Fill=i32Arg2;

	if (i32Arg1+i32Fill>getSize())
		i32Fill=getSize()-i32Arg1;

	FillMemory(getBufferPointer()+i32Arg1,i32Fill,u8Arg);

	return true;

}

BUF D2CBuffer::getBuffer() const { return getBufferPointer(); }

bool D2CBuffer::resize(I32 i32Arg) {

	if (i32Arg<0)
		return false;

	if (i32Arg==0)
		return reset(0);

	if (i32Arg==getSize())
		return true;

	if (getSize()<1)
		return reset(i32Arg);

	BUF bfTmp=(BUF)D2Malloc(i32Arg+1);

	if (bfTmp==NULL)
		return false;

	if (i32Arg>getSize())
		D2MemCopy(bfTmp,getBufferPointer(),getSize());
	else
		D2MemCopy(bfTmp,getBufferPointer(),i32Arg);

	D2Free(getBufferPointer());
	setBufferPointer(bfTmp);
	setSize(i32Arg);

	return true;

}

bool D2CBuffer::clear(I32 i32Arg,UI8 u8Arg) {

	if (i32Arg<0)
		return false;

	if (!reset(i32Arg))
		return false;

	if (i32Arg==0)
		return true;

	return D2FillMemory(getBufferPointer(),getSize(),u8Arg);

}

bool D2CBuffer::append(UI8 u8Arg) {

	if (!resize(getSize()+1))
		return false;

	return write(getLastIndex(),u8Arg);

}

bool D2CBuffer::append(const BUF bfArg,I32 i32Arg) {

	I32 i32Last=getLastIndex()+1;

	if (!resize(getSize()+i32Arg))
		return false;

	write(i32Last,bfArg,i32Arg);

	return true;

}

bool D2CBuffer::append(const D2IBufferReader *pbrArg) {

	I32 i32Last=getLastIndex()+1;

	if (pbrArg==NULL || !resize(getSize()+pbrArg->getSize()))
		return false;

	if (pbrArg->getSize()==0)
		return true;

	BUF bfTmp=pbrArg->copy();

	if (bfTmp==NULL)
		return false;

	write(i32Last,bfTmp,pbrArg->getSize());

	D2Free(bfTmp);
	return true;

}

bool D2CBuffer::insert(I32 i32Arg1,const BUF bfArg,I32 i32Arg2) {

	if (i32Arg1<0 || bfArg==NULL || i32Arg2<1)
		return false;

	if (i32Arg1>getLastIndex() && resize(i32Arg1+i32Arg2)) {

		write(i32Arg1,bfArg,i32Arg2);
		return true;

	}

	if (i32Arg1<=getLastIndex() && resize(getSize()+i32Arg2)) {

		BUF bf1=getBufferPointer()+i32Arg1,bf2=bf1+i32Arg2;

		D2MemCopy(bf2,bf1,getSize()-i32Arg1);

		write(i32Arg1,bfArg,i32Arg2);
		return true;

	}

	return false;

}

bool D2CBuffer::insert(I32 i32Arg1,const D2IBufferReader *pbrArg) {

	if (i32Arg1<0 || pbrArg==NULL || pbrArg->getSize()<1)
		return false;

	BUF bfTmp=pbrArg->copy();

	if (insert(i32Arg1,bfTmp,pbrArg->getSize())) {

		D2Free(bfTmp);
		return true;

	} else {
		 
		D2Free(bfTmp);
		return false;

	}

}

bool D2CBuffer::reset(I32 i32Arg) {

	if (i32Arg<0)
		return false;

	if (i32Arg==getSize()) {

		D2FillMemory(getBufferPointer(),getSize(),0);
		return true;

	}

	if (getBufferPointer()!=NULL)
		D2Free(getBufferPointer());

	setBufferPointer(NULL);

	if (i32Arg==0) {

		setSize(0);
		return true;

	}

	setBufferPointer((BUF)D2Malloc(i32Arg+1));

	if (getBufferPointer()==NULL) {

		setSize(0);
		return false;

	}

	setSize(i32Arg);

	return true;

}

bool D2CBuffer::setSize(I32 i32Arg) {

	m_i32BufferSize=i32Arg;

	return true;

}

BUF D2CBuffer::getBufferPointer() const { return m_bfBuffer; }

bool D2CBuffer::setBufferPointer(BUF bfArg) {

	m_bfBuffer=bfArg;

	return true;

}

I32 D2CBuffer::getLastIndex() const { return getSize()-1; }
