#include "windef.h"
#include "interfaces.h"
#include <stdexcept>

class D2CFixedBuffer:public D2IObject,public D2IFixedBuffer {

public:

	D2CFixedBuffer();
	D2CFixedBuffer(I32);
	D2CFixedBuffer(const BUF,I32);
	D2CFixedBuffer(const D2IBufferReader *);
	D2CFixedBuffer(const D2CFixedBuffer&);

	D2CFixedBuffer& operator=(const D2CFixedBuffer&);

	virtual ~D2CFixedBuffer();

	//D2IObject
	virtual UI32 getClassID() const;
	virtual UI32 getObjectType() const;
	virtual UI8 getObjectVersion() const;
	virtual UI8 getDataType() const;
	virtual UI16 getEncoding() const;
	virtual void * getInterface(UI32);
	virtual CSTR getCSTR();

	// D2IBufferInterface
	virtual I32 getSize() const;

	// D2IBufferReader
	virtual BUF copy() const;
	virtual UI8 read(I32) const throw(std::range_error);
	virtual I32 read(I32,BUF,I32) const;
	virtual bool equal(const BUF,I32) const;
	virtual bool equal(const D2IBufferReader *) const;

	// D2IBufferWriter
	virtual bool write(I32,UI8);
	virtual I32 write(I32,const BUF,I32);
	virtual bool fill(I32,UI8,I32);

protected:

	virtual bool reset(I32);

	virtual bool setSize(I32);

	virtual BUF getBufferPointer() const;
	virtual bool setBufferPointer(const BUF);

	virtual BUF getCSTRPointer() const;
	virtual bool setCSTRPointer(const BUF);

private:

	I32 m_i32BufferSize;
	BUF m_bfBuffer,m_bfCSTR;
	bool m_isAllocate;

};

class D2CBuffer:public D2IBuffer {

public:

	D2CBuffer();
	D2CBuffer(I32);
	D2CBuffer(BUF,I32);
	D2CBuffer(const D2IBufferReader *);
	D2CBuffer(const D2CBuffer&);

	D2CBuffer& operator=(const D2CBuffer&);

	virtual ~D2CBuffer();

	// D2IObject
	virtual UI32 getClassID() const;
	virtual UI32 getObjectType() const;
	virtual UI8 getObjectVersion() const;
	virtual UI8 getDataType() const;
	virtual UI16 getEncoding() const;
	virtual void * getInterface(UI32);
	virtual CSTR getCSTR();

	// D2IBufferInterface
	virtual I32 getSize() const;

	// D2IBufferReader
	virtual BUF copy() const;
	virtual UI8 read(I32) const throw(std::range_error);
	virtual I32 read(I32,BUF,I32) const;
	virtual bool equal(const BUF,I32) const;
	virtual bool equal(const D2IBufferReader *) const;

	// D2IBufferWriter
	virtual bool write(I32,UI8);
	virtual I32 write(I32,const BUF,I32);
	virtual bool fill(I32,UI8,I32);

	//D2IBufferAdministrator
	virtual BUF getBuffer() const;
	virtual bool resize(I32);
	virtual bool clear(I32,UI8);
	virtual bool assign(const D2IBufferReader *);

	// D2IBuffer
	virtual bool append(UI8);
	virtual bool append(const BUF,I32);
	virtual bool append(const D2IBufferReader *);
	virtual bool insert(I32,const BUF,I32);
	virtual bool insert(I32,const D2IBufferReader *);

protected:

	virtual bool reset(I32);

	virtual bool setSize(I32);

	virtual BUF getBufferPointer() const;
	virtual bool setBufferPointer(const BUF);
	virtual I32 getLastIndex() const;

private:

	I32 m_i32BufferSize;
	BUF m_bfBuffer;

};

class D2CFileReaderStream:public D2IObject,public D2IStreamReader {

public:

	D2CFileReaderStream(const CSTR);

	virtual bool seek(I32);
	virtual bool close();

	virtual ~D2CFileReaderStream();

	// D2IObject
	virtual UI32 getClassID() const;
	virtual UI32 getObjectType() const;
	virtual UI8 getObjectVersion() const;
	virtual UI8 getDataType() const;
	virtual UI16 getEncoding() const;
	virtual void * getInterface(UI32);
	virtual CSTR getCSTR();

	// D2IStreamReader;
	virtual UI8 read();
	virtual I32 read(BUF,I32);
	virtual I32 read(D2IBufferWriter *);

private:

	FILE *m_fp;

};

class D2CFileWriterStream:public D2IObject,public D2IStreamWriter {

public:

	D2CFileWriterStream(CSTR);

	virtual bool seek(I32);
	virtual bool close();

	virtual ~D2CFileWriterStream();

	// D2IObject
	virtual UI32 getClassID() const;
	virtual UI32 getObjectType() const;
	virtual UI8 getObjectVersion() const;
	virtual UI8 getDataType() const;
	virtual UI16 getEncoding() const;
	virtual void * getInterface(UI32);
	virtual CSTR getCSTR();

	// D2IStreamWriter
	virtual bool write(UI8);
	virtual I32 write(const BUF,I32);
	virtual I32 write(const D2IBufferReader *);

private:

	FILE *m_fp;

};

class D2CDIB32:public D2IObject,public D2ISerialize,public D2IDeserialize,public D2IBitmap32XRGB8 {

public:

	D2CDIB32();
	D2CDIB32(I32,I32);
	D2CDIB32(const D2CDIB32&);

	D2CDIB32& operator=(const D2CDIB32&);
	bool assign(const D2IBitmapReader *);

	~D2CDIB32();

	//D2IObject
	virtual UI32 getClassID() const;
	virtual UI32 getObjectType() const;
	virtual UI8 getObjectVersion() const;
	virtual UI8 getDataType() const;
	virtual UI16 getEncoding() const;
	virtual void * getInterface(UI32);
	virtual CSTR getCSTR();

	// D2ISerialize
	virtual bool serialize(D2IStreamWriter *,UI32);
	virtual bool deserialize(D2IStreamReader *,UI32);

	// D2IBitmapInterface
	virtual I32 getWidth() const;
	virtual I32 getHeight() const;
	virtual UI32 getOrigin() const;

	// D2IBitmapReader
	virtual UI32 getRGB24(I32,I32) const throw(std::out_of_range);
	virtual UI8 getR8(I32,I32) const throw(std::out_of_range);
	virtual UI8 getG8(I32,I32) const throw(std::out_of_range);
	virtual UI8 getB8(I32,I32) const throw(std::out_of_range);
	virtual I32 getColors() const;

	// D2IBitmapWriter
	virtual bool setRGB24(I32,I32,UI32);
	virtual bool setR8(I32,I32,UI8);
	virtual bool setG8(I32,I32,UI8);
	virtual bool setB8(I32,I32,UI8);

	virtual bool reset(I32,I32);
	virtual bool clear(I32,I32,I32);

	//D2IBitmap32XRGB8
	virtual void * getHeader() const;
	virtual void * getPixels() const;

	virtual UI32 getXRGB32(I32,I32) const throw(std::out_of_range);
	virtual bool setXRGB32(I32,I32,UI32);

protected:

	virtual void * getHeaderPointer() const;
	virtual UI32 * getPixelsPointer() const;
	virtual bool setSize(I32,I32);
	virtual bool setWidth(I32);
	virtual bool setHeight(I32);
	virtual bool saveBMP(D2IStreamWriter *,UI32);
	virtual bool loadBMP(D2IStreamReader *,UI32);

private:

	CSTR m_csStr;
	D2CBuffer *m_pbuDIB;
	D2CFixedBuffer *m_pbInfo,*m_pbPixels;
	UI32 *m_pu32Pixels;
	void *m_pvHeader;

	I32 m_i32Width,m_i32Height;

};


