#ifndef _interfaces_
#define _interfaces_

#include "windef.h"

class D2IObject {

public:

	virtual UI32 getClassID() const=0;
	virtual UI32 getObjectType() const=0;
	virtual UI8 getObjectVersion() const=0;
	virtual UI8 getDataType() const=0;
	virtual UI16 getEncoding() const=0;
	virtual void * getInterface(UI32)=0;
	virtual CSTR getCSTR()=0;

	virtual ~D2IObject() {}

};

class D2IBufferInterface {

public:

	virtual I32 getSize() const=0;

	virtual ~D2IBufferInterface() {}

};

class D2IBufferReader:public D2IBufferInterface {

public:

	virtual BUF copy() const=0;
	virtual UI8 read(I32) const=0;
	virtual I32 read(I32,BUF,I32) const=0;
	virtual bool equal(const BUF,I32) const=0;
	virtual bool equal(const D2IBufferReader *) const=0;

	~D2IBufferReader() {}

};

class D2IBufferWriter:public D2IBufferInterface {

public:

	virtual bool write(I32,UI8)=0;
	virtual I32 write(I32,const BUF,I32)=0;
	virtual bool fill(I32,UI8,I32)=0;

	~D2IBufferWriter() {}

};

class D2IFixedBuffer:public D2IBufferReader,public D2IBufferWriter {

public:

	virtual ~D2IFixedBuffer() {}

};

class D2IBufferAdministrator:public D2IBufferInterface {

public:

	virtual BUF getBuffer() const=0;
	virtual bool resize(I32)=0;
	virtual bool clear(I32,UI8)=0;
	virtual bool assign(const D2IBufferReader *)=0;

	virtual ~D2IBufferAdministrator() {}

};

class D2IBuffer:public D2IFixedBuffer,public D2IBufferAdministrator {

public:

	virtual bool append(UI8)=0;
	virtual bool append(const BUF,I32)=0;
	virtual bool append(const D2IBufferReader *)=0;
	virtual bool insert(I32,const BUF,I32)=0;
	virtual bool insert(I32,const D2IBufferReader *)=0;

	virtual ~D2IBuffer() {}

};

class D2IStreamReader {

public:

	virtual UI8 read()=0;
	virtual I32 read(BUF,I32)=0;
	virtual I32 read(D2IBufferWriter *)=0;

	virtual ~D2IStreamReader() {}

};

class D2IStreamWriter {

public:

	virtual bool write(UI8)=0;
	virtual I32 write(const BUF,I32)=0;
	virtual I32 write(const D2IBufferReader *)=0;

	virtual ~D2IStreamWriter() {};

};

class D2IStream:public D2IStreamReader,public D2IStreamWriter {

public:

	~D2IStream() {}

};

class D2ISerialize {

public:

	virtual bool serialize(D2IStreamWriter *,UI32)=0;

	~D2ISerialize() {}

};

class D2IDeserialize {

public:

	virtual bool deserialize(D2IStreamReader *,UI32)=0;

	~D2IDeserialize() {}

};

class D2IBitmapInterface {

public:

	virtual I32 getWidth() const=0;
	virtual I32 getHeight() const=0;
	virtual UI32 getOrigin() const=0;

	virtual ~D2IBitmapInterface() {}

};

class D2IBitmapReader:public D2IBitmapInterface {

public:

	virtual UI32 getRGB24(I32,I32) const=0;
	virtual UI8 getR8(I32,I32) const=0;
	virtual UI8 getG8(I32,I32) const=0;
	virtual UI8 getB8(I32,I32) const=0;
	virtual I32 getColors() const=0;

	virtual ~D2IBitmapReader() {}

};

class D2IBitmapWriter:public D2IBitmapInterface {

public:

	virtual bool setRGB24(I32,I32,UI32)=0;
	virtual bool setR8(I32,I32,UI8)=0;
	virtual bool setG8(I32,I32,UI8)=0;
	virtual bool setB8(I32,I32,UI8)=0;

	virtual bool clear(I32,I32,I32)=0;

	virtual ~D2IBitmapWriter() {}

};

class D2IBitmap:public D2IBitmapReader,public D2IBitmapWriter {

public:

	virtual bool reset(I32,I32)=0;

	virtual void * getHeader() const=0;
	virtual void * getPixels() const=0;

	virtual ~D2IBitmap() {}

};

class D2IBitmap32XRGB8:public D2IBitmap {

public:

	virtual UI32 getXRGB32(I32,I32) const=0;
	virtual bool setXRGB32(I32,I32,UI32)=0;

	virtual ~D2IBitmap32XRGB8() {}

};

#endif
